@@ -160,12 +160,16 @@ typedef union
    to whatever platform routine is available, or to our own fallback
    implementation.  */
 #define INCLUDE_explicit_bzero 0
-#if defined HAVE_EXPLICIT_BZERO
+#if defined HAVE_MEMSET_EXPLICIT
+/* Preferred over explicit_bzero, as this is part of the C23 standard.
+   See: ISO/IEC 9899:2024  */
+#define explicit_bzero(s, len) memset_explicit(s, 0, len)
+#elif defined HAVE_MEMSET_S
+#define explicit_bzero(s, len) memset_s(s, len, 0, len)
+#elif defined HAVE_EXPLICIT_BZERO
 /* nothing to do */
 #elif defined HAVE_EXPLICIT_MEMSET
 #define explicit_bzero(s, len) explicit_memset(s, 0, len)
-#elif defined HAVE_MEMSET_S
-#define explicit_bzero(s, len) memset_s(s, len, 0, len)
 #else
 /* activate our fallback implementation */
 #undef INCLUDE_explicit_bzero
@@ -201,6 +205,11 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
   __asm__(".globl _" extstr);                           \
   __asm__(".set _" extstr ", _" #intname)
 
+#elif defined _WIN32
+
+/* .symver is only supported for ELF format, Windows uses COFF/PE */
+# define symver_set(extstr, intname, version, mode)
+
 #elif defined __GNUC__ && __GNUC__ >= 3
 
 # define _strong_alias(name, aliasname) \
@@ -208,7 +217,7 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
 
 /* Starting with GCC 10, we can use the symver attribute, which is also
    needed at the point we decide to enable link-time optimization.  */
-# if __GNUC__ >= 10
+# if defined HAVE_FUNC_ATTRIBUTE_SYMVER
 
 /* Set the symbol version for EXTNAME, which uses INTNAME as its
    implementation.  */
@@ -239,7 +248,7 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
 
 /* The macros for versioned symbols work differently in this library
    than they do in glibc.  They are mostly auto-generated
-   (see build-aux/gen-crypt-symbol-vers-h)
+   (see build-aux/scripts/gen-crypt-symbol-vers-h)
    and we currently don't support compatibility symbols that need a different
    definition from the default version.
 
@@ -357,11 +366,13 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
 #define libcperciva_SHA512_Buf    _crypt_SHA512_Buf
 #endif
 
-#if INCLUDE_md5crypt || INCLUDE_sha256crypt || INCLUDE_sha512crypt
+#if INCLUDE_md5crypt || INCLUDE_sha256crypt || INCLUDE_sha512crypt || \
+    INCLUDE_sm3crypt
 #define gensalt_sha_rn           _crypt_gensalt_sha_rn
 #endif
 
-#if INCLUDE_yescrypt || INCLUDE_scrypt || INCLUDE_gost_yescrypt
+#if INCLUDE_yescrypt || INCLUDE_scrypt || INCLUDE_gost_yescrypt || \
+    INCLUDE_sm3_yescrypt
 #define PBKDF2_SHA256            _crypt_PBKDF2_SHA256
 #define crypto_scrypt            _crypt_crypto_scrypt
 #define yescrypt                 _crypt_yescrypt
@@ -385,13 +396,21 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
 #endif
 
 #if INCLUDE_sha256crypt || INCLUDE_scrypt || INCLUDE_yescrypt || \
-    INCLUDE_gost_yescrypt
+    INCLUDE_gost_yescrypt || INCLUDE_sm3_yescrypt
 #define libcperciva_SHA256_Init  _crypt_SHA256_Init
 #define libcperciva_SHA256_Update _crypt_SHA256_Update
 #define libcperciva_SHA256_Final _crypt_SHA256_Final
 #define libcperciva_SHA256_Buf   _crypt_SHA256_Buf
 #endif
 
+#if INCLUDE_sm3crypt || INCLUDE_sm3_yescrypt
+#define sm3_init   _crypt_sm3_init
+#define sm3_update _crypt_sm3_update
+#define sm3_final  _crypt_sm3_final
+#define sm3_hash   _crypt_sm3_hash
+#define sm3_buf    _crypt_sm3_buf
+#endif
+
 #if INCLUDE_gost_yescrypt
 #define GOST34112012Init       _crypt_GOST34112012_Init
 #define GOST34112012Update     _crypt_GOST34112012_Update
@@ -399,15 +418,23 @@ extern size_t strcpy_or_abort (void *dst, size_t d_size, const void *src);
 #define GOST34112012Cleanup    _crypt_GOST34112012_Cleanup
 #define gost_hash256           _crypt_gost_hash256
 #define gost_hmac256           _crypt_gost_hmac256
+#endif
 
-/* Those are not present, if gost-yescrypt is selected,
-   but yescrypt is not. */
-#if !INCLUDE_yescrypt
+#if INCLUDE_sm3_yescrypt
+#define sm3_hmac_buf      _crypt_sm3_hmac_buf
+#define sm3_hmac_final    _crypt_sm3_hmac_final
+#define sm3_hmac_init     _crypt_sm3_hmac_init
+#define sm3_hmac_update   _crypt_sm3_hmac_update
+#define sm3_hmac          _crypt_sm3_hmac
+#endif
+
+/* Those are not present, if gost-yescrypt or sm3-yescrypt
+   is selected, but yescrypt is not. */
+#if !INCLUDE_yescrypt && (INCLUDE_gost_yescrypt || INCLUDE_sm3_yescrypt)
 #define gensalt_yescrypt_rn _crypt_gensalt_yescrypt_rn
 extern void gensalt_yescrypt_rn
 (unsigned long, const uint8_t *, size_t, uint8_t *, size_t);
 #endif
-#endif
 
 /* Those are not present, if des-big is selected, but des is not. */
 #if INCLUDE_bigcrypt && !INCLUDE_descrypt
@@ -439,7 +466,7 @@ extern bool get_random_bytes (void *buf, size_t buflen);
 
 /* Generate a setting string in the format common to md5crypt,
    sha256crypt, and sha512crypt.  */
-extern void gensalt_sha_rn (char tag, size_t maxsalt, unsigned long defcount,
+extern void gensalt_sha_rn (const char *tag, size_t maxsalt, unsigned long defcount,
                             unsigned long mincount, unsigned long maxcount,
                             unsigned long count,
                             const uint8_t *rbytes, size_t nrbytes,
